@@ -1,10 +1,10 @@
 #############################################################################
 #
-#	$Id: protocol.py,v 2.94.2.24 2008/10/05 00:02:18 irmen Exp $
+#	$Id: protocol.py,v 2.94.2.40 2010/05/22 16:31:10 irmen Exp $
 #	Pyro Protocol Adapters
 #
 #	This is part of "Pyro" - Python Remote Objects
-#	which is (c) Irmen de Jong - irmen@users.sourceforge.net
+#	which is (c) Irmen de Jong - irmen@razorvine.net
 #
 #############################################################################
 
@@ -18,8 +18,8 @@ except ImportError:
 	md5=md5.md5
 import Pyro.constants, Pyro.util
 
-from errors import *
-from errors import _InternalNoModuleError
+from Pyro.errors import *
+from Pyro.errors import _InternalNoModuleError
 pickle = Pyro.util.getPickle()
 Log = Pyro.util.Log
 	
@@ -185,8 +185,8 @@ PFLG_CHECKSUM =   0x02		# protocol flag: checksum body
 PFLG_XMLPICKLE_GNOSIS =  0x04		# protocol flag: used xml pickling (Gnosis)
 
 
-class PYROAdapter:
-	headerFmt = '!4sHHlHl'	# version 4 header (id, ver, hsiz,bsiz,pflags,crc)
+class PYROAdapter(object):
+	headerFmt = '!4sHHlHl'	# header (id, ver, hsiz,bsiz,pflags,crc)
 	headerID = 'PYRO'
 	connectMSG='CONNECT'
 	acceptMSG= 'GRANTED'
@@ -196,7 +196,7 @@ class PYROAdapter:
 	AUTH_CHALLENGE_SIZE = 16
 
 	headerSize = struct.calcsize(headerFmt)
-	version=4				# version 4 protocol
+	version=5				# version 5 protocol
 	def __init__(self):
 		self.onewayMethods=[]		# methods that should be called one-way
 		self.timeout=None			# socket timeout
@@ -210,10 +210,10 @@ class PYROAdapter:
 	def __del__(self):
 		self.release(nolog=1)
 	def __getstate__(self):
-		# need to tweak the pickle because lock objects can't be pickled
+		# need to tweak the pickle because lock objects and conn objects can't be pickled
+		self.release()   # cannot pickle the active connection so just release it
 		d=self.__dict__.copy()
 		del d["lock"]
-		del d["bindlock"]
 		return d
 	def __setstate__(self, state):
 		# restore the pickle state and recreate the unpickleable lock objects
@@ -221,7 +221,6 @@ class PYROAdapter:
 		self.__getLockObjects()
 	def __getLockObjects(self):
 		self.lock=Pyro.util.getLockObject()
-		self.bindlock=Pyro.util.getLockObject()
 	def recvAuthChallenge(self, conn):
 		ver,body,pflags = self.receiveMsg(conn)
 		if ver==self.version and len(body)==self.AUTH_CHALLENGE_SIZE:
@@ -239,9 +238,9 @@ class PYROAdapter:
 			Log.error('PYROAdapter','incompatible protocol in URI:',URI.protocol)
 			raise ProtocolError('incompatible protocol in URI')
 		try:
-			self.bindlock.acquire()   # only 1 thread at a time can bind the URI
+			self.lock.acquire()   # only 1 thread at a time can bind the URI
 			try:
-				self.URI=URI.clone()
+				self.URI=URI
 				sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 				sock.connect((URI.address, URI.port))
 				conn=TCPConnection(sock,sock.getpeername())
@@ -271,7 +270,7 @@ class PYROAdapter:
 				Log.msg('PYROAdapter','connection failed to URI',str(URI))
 				raise ProtocolError('connection failed')
 		finally:
-			self.bindlock.release()
+			self.lock.release()
 
 	def resolvePYROLOC_URI(self, newProtocol):
 		# This method looks up the object URI referenced by
@@ -280,7 +279,7 @@ class PYROAdapter:
 		Log.msg('PYROAdapter','resolving PYROLOC name: ',objectName)
 		# call the special Resolve method on the daemon itself:
 		self.URI.objectID=Pyro.constants.INTERNAL_DAEMON_GUID
-		result=self.remoteInvocation('ResolvePYROLOC',0,objectName)
+		result=self._remoteInvocation('ResolvePYROLOC',0,objectName)
 		# found it, switching to regular pyro protocol
 		self.URI.objectID=result
 		self.URI.protocol=newProtocol
@@ -350,6 +349,7 @@ class PYROAdapter:
 
 	# Retrieve code from the remote peer. Works recursively.
 	def _retrieveCode(self, mname, level):
+		Log.msg("PYROAdapter","client cannot find module: "+mname)
 		# XXX this is nasty code, and also duplicated in core.py remote_supply_code()
 		if mname in sys.modules:
 			# module is already loaded, do nothing
@@ -377,7 +377,8 @@ class PYROAdapter:
 					setattr(mod, m, new.module(real_path))
 					mod = getattr(mod, m)
 					sys.modules[real_path] = mod
-					
+
+			Log.msg('PYROAdapter','loading supplied code: ',mname)
 			if module[0:4] != imp.get_magic():
 				code = compile(module, "<downloaded>", "exec")
 			else:
@@ -421,7 +422,15 @@ class PYROAdapter:
 		ver,answer,pflags = self.receiveMsg(self.conn,1)
 		if answer is None:
 			raise ProtocolError('incorrect answer received')
-		return pickle.loads(answer)
+		answer=pickle.loads(answer)
+		if isinstance(answer,PyroExceptionCapsule):
+			if isinstance(answer.excObj,_InternalNoModuleError):
+				# server couldn't load module, supply it
+				return self.processMissingModuleError(answer.excObj, method, flags, args)
+			else:
+				# we have an encapsulated exception, raise it again.
+				answer.raiseEx()
+		return answer
 
 	def remoteInvocation(self, method, flags, *args):
 		try:
@@ -483,52 +492,58 @@ class PYROAdapter:
 
 		if isinstance(answer,PyroExceptionCapsule):
 			if isinstance(answer.excObj,_InternalNoModuleError):
-				# server couldn't load module, supply it
-				# XXX this code is ugly. and duplicated in remote_retrieve_code in core.py
-				try:
-					importmodule=new.module('-agent-import-')
-					mname=answer.excObj.modulename
-					# not used: fromlist=answer.excObj.fromlist
-					try:
-						exec 'import '+mname in importmodule.__dict__
-					except ImportError:
-						Log.error('PYROAdapter','Server wanted a non-existing module:',mname)
-						raise PyroError('Server wanted a non-existing module',mname)
-					m=eval('importmodule.'+mname)
-					bytecode=None
-					if hasattr(m,"_PYRO_bytecode"):
-						# use the bytecode that was put there earlier,
-						# this avoids recompiles of the source .py if we don't have .pyc bytecode available
-						bytecode=m._PYRO_bytecode
-					else:
-						# try to load the module's compiled source, or the real .py source if that fails.
-						# note that the source code (.py) is opened with universal newline mode
-						if not hasattr(m,"__file__"):
-							raise PyroError("cannot read module source code",mname)
-						(filebase,ext)=os.path.splitext(m.__file__)
-						if ext.startswith(".PY"):
-							exts = ( (".PYO","rb"), (".PYC","rb"), (".PY","rU") )	# uppercase
-						else:
-							exts = ( (".pyo","rb"), (".pyc","rb"), (".py","rU") )	# lowercase
-						for ext,mode in exts:
-							try:
-								bytecode=open(filebase+ext, mode).read()
-								break
-							except EnvironmentError:
-								pass
-					if bytecode:
-						self._remoteInvocationMobileCode("remote_supply_code",0,mname, bytecode, self.conn.sock.getsockname())
-						# retry the method invocation
-						return self._remoteInvocation(* (method, flags)+args)   # use the non-locking call
-					Log.error("PYROAdapter","cannot read module source code for module:", mname)
-					raise PyroError("cannot read module source code",mname)
-				finally:
-					del importmodule
+				# server couldn't load the module, send it
+				return self.processMissingModuleError(answer.excObj, method, flags, args)
 			else:
 				# we have an encapsulated exception, raise it again.
 				answer.raiseEx()
 		return answer
 
+	def processMissingModuleError(self, errorinfo, method, flags, args):
+		# server couldn't load module, supply it
+		# XXX this code is ugly. and duplicated in remote_retrieve_code in core.py
+		Log.msg('PYROAdapter',"server can't load module: "+errorinfo.modulename)
+		try:
+			importmodule=new.module('-agent-import-')
+			mname=errorinfo.modulename
+			# not used: fromlist=errorinfo.fromlist
+			try:
+				exec 'import '+mname in importmodule.__dict__
+			except ImportError:
+				Log.error('PYROAdapter','Server wanted a non-existing module:',mname)
+				raise PyroError('Server wanted a non-existing module',mname)
+			m=eval('importmodule.'+mname)
+			bytecode=None
+			if hasattr(m,"_PYRO_bytecode"):
+				# use the bytecode that was put there earlier,
+				# this avoids recompiles of the source .py if we don't have .pyc bytecode available
+				bytecode=m._PYRO_bytecode
+			else:
+				# try to load the module's compiled source, or the real .py source if that fails.
+				# note that the source code (.py) is opened with universal newline mode
+				if not hasattr(m,"__file__"):
+					raise PyroError("cannot read module source code",mname)
+				(filebase,ext)=os.path.splitext(m.__file__)
+				if ext.startswith(".PY"):
+					exts = ( (".PYO","rb"), (".PYC","rb"), (".PY","rU") )	# uppercase
+				else:
+					exts = ( (".pyo","rb"), (".pyc","rb"), (".py","rU") )	# lowercase
+				for ext,mode in exts:
+					try:
+						bytecode=open(filebase+ext, mode).read()
+						break
+					except EnvironmentError:
+						pass
+			if bytecode:
+				Log.msg('PYROAdapter',"sending module to server: "+mname)
+				self._remoteInvocationMobileCode("remote_supply_code",0,mname, bytecode, self.conn.sock.getsockname())
+				# retry the method invocation
+				return self._remoteInvocation(* (method, flags)+args)   # use the non-locking call
+			Log.error("PYROAdapter","cannot read module source code for module:", mname)
+			raise PyroError("cannot read module source code",mname)
+		finally:
+			del importmodule
+		
 	# (private) receives a socket message, returns: (protocolver, message, protocolflags)
 	def receiveMsg(self,conn,noReply=0):
 		msg=sock_recvmsg(conn.sock, self.headerSize, self.timeout)
@@ -636,16 +651,19 @@ class PYROAdapter:
 			return
 		else:
 			# Do the invocation. We are already running in our own thread.
-			if req[2]&Pyro.constants.RIF_Oneway and daemon.threaded:   # flags
+			if req[2]&Pyro.constants.RIF_Oneway and Pyro.config.PYRO_ONEWAY_THREADED and daemon.threaded:
 				# received a oneway call, run this in its own thread.
-				thread=Thread(target=self._handleInvocation2, args=(daemon,req,pflags,conn,o))
+				thread=Thread(target=self._handleInvocation2, args=(daemon,req,pflags,conn,o,True))
 				thread.setDaemon(1)   # thread must exit at program termination.
+				thread.localStorage=LocalStorage()   # set local storage for the new thread
 				thread.start()
 			else:
 				# not oneway or not in threaded mode, just do the invocation synchronously
-				self._handleInvocation2(daemon,req,pflags,conn,o)
+				self._handleInvocation2(daemon,req,pflags,conn,o,False)
 
-	def _handleInvocation2(self, daemon, req, pflags, conn, obj):
+	def _handleInvocation2(self, daemon, req, pflags, conn, obj, mustInitTLS=False):
+		if mustInitTLS:
+			daemon.initTLS(daemon.getLocalStorage())
 		try:
 			flags=req[2]
 			importer=None
@@ -746,15 +764,18 @@ class PYROAdapter:
 			return 0
 
 # import wrapper class to help with importing remote modules
-class agent_import:
+class agent_import(object):
 	def __init__(self, orig_import):
 		self.orig_import=orig_import
-	def __call__(self,name, globals={},locals={},fromlist=None, *rest):
+	def __call__(self,name,iglobals={},ilocals={},fromlist=None, *rest, **krest):
+		if os.name=="java":
+			# workaround for odd Jython bug, iglobals and ilocals may not exist in this scope...(?!)
+			iglobals=vars().get("iglobals",{})
+			ilocals=vars().get("ilocals",{})
 		# save the import details:
 		self.name=name		# note: this must be a str object
 		self.fromlist=fromlist
-		return self.orig_import(name,globals,locals,fromlist, *rest)
-
+		return self.orig_import(name,iglobals,ilocals,fromlist, *rest, **krest)
 
 #
 # The SSL adapter that handles SSL connections instead of regular sockets.
@@ -768,7 +789,12 @@ class PYROSSLAdapter(PYROAdapter):
 			raise ProtocolError('SSL not available')
 
 		self.ctx = SSL.Context('sslv23')
-		self.ctx.load_cert(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CLIENT_CERT))
+		if Pyro.config.PYROSSL_KEY:
+			keyfile = os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_KEY)
+		else:
+			keyfile = None
+		self.ctx.load_cert(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CERT),
+				   keyfile)
 		self.ctx.load_client_ca(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CA_CERT))
 		self.ctx.load_verify_info(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CA_CERT))
 		self.ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,10)
@@ -783,30 +809,34 @@ class PYROSSLAdapter(PYROAdapter):
 			Log.error('PYROSSLAdapter','incompatible protocol in URI:',URI.protocol)
 			raise ProtocolError('incompatible protocol in URI')
 		try:
-			self.URI=URI.clone()
-			sock = SSL.Connection(self.ctx,socket.socket(socket.AF_INET, socket.SOCK_STREAM))
-			if not Pyro.config.PYROSSL_POSTCONNCHECK:
-				sock.postConnectionCheck=None
-			sock.connect((URI.address, URI.port))
-			conn=TCPConnection(sock, sock.getpeername())
-			# receive the authentication challenge string, and use that to build the actual identification string.
-			authChallenge=self.recvAuthChallenge(conn)
-			# reply with our ident token, generated from the ident passphrase and the challenge
-			msg = self._sendConnect(sock,self.newConnValidator.createAuthToken(self.ident, authChallenge, conn.addr, self.URI, None) )
-			if msg==self.acceptMSG:
-				self.conn=conn
-				self.conn.connected=1
-				Log.msg('PYROSSLAdapter','connected to',str(URI))
-				if URI.protocol=='PYROLOCSSL':
-					self.resolvePYROLOC_URI("PYROSSL") # updates self.URI
-			elif msg[:len(self.denyMSG)]==self.denyMSG:
-				try:
-					raise ConnectionDeniedError(Pyro.constants.deniedReasons[int(msg[-1])])
-				except (KeyError,ValueError):
-					raise ConnectionDeniedError('invalid response')
-		except socket.error:
-			Log.msg('PYROSSLAdapter','connection failed to URI',str(URI))
-			raise ProtocolError('connection failed')
+			self.lock.acquire()   # only 1 thread at a time can bind the URI
+			try:
+				self.URI=URI
+				sock = SSL.Connection(self.ctx,socket.socket(socket.AF_INET, socket.SOCK_STREAM))
+				if not Pyro.config.PYROSSL_POSTCONNCHECK:
+					sock.postConnectionCheck=None
+				sock.connect((URI.address, URI.port))
+				conn=TCPConnection(sock, sock.getpeername())
+				# receive the authentication challenge string, and use that to build the actual identification string.
+				authChallenge=self.recvAuthChallenge(conn)
+				# reply with our ident token, generated from the ident passphrase and the challenge
+				msg = self._sendConnect(sock,self.newConnValidator.createAuthToken(self.ident, authChallenge, conn.addr, self.URI, None) )
+				if msg==self.acceptMSG:
+					self.conn=conn
+					self.conn.connected=1
+					Log.msg('PYROSSLAdapter','connected to',str(URI))
+					if URI.protocol=='PYROLOCSSL':
+						self.resolvePYROLOC_URI("PYROSSL") # updates self.URI
+				elif msg[:len(self.denyMSG)]==self.denyMSG:
+					try:
+						raise ConnectionDeniedError(Pyro.constants.deniedReasons[int(msg[-1])])
+					except (KeyError,ValueError):
+						raise ConnectionDeniedError('invalid response')
+			except socket.error:
+				Log.msg('PYROSSLAdapter','connection failed to URI',str(URI))
+				raise ProtocolError('connection failed')
+		finally:
+			self.lock.release()
 
 	def _sendConnect(self, sock, ident):
 		return PYROAdapter._sendConnect(self, sock, ident)
@@ -823,7 +853,7 @@ def getProtocolAdapter(protocol):
 
 
 #-------- TCPConnection object for TCPServer class
-class TCPConnection:
+class TCPConnection(object):
 	def __init__(self, sock, addr):
 		self.sock = sock
 		set_sock_keepalive(self.sock)   # enable tcp/ip keepalive on this socket
@@ -848,7 +878,7 @@ class TCPConnection:
 #-------- DefaultConnValidator checks max number of connections & identification
 #-------- and ident check is done using hmac-md5 secure hash of passphrase+challenge.
 #-------- Contains client- & server-side auth code.
-class DefaultConnValidator:
+class DefaultConnValidator(object):
 	def __init__(self):
 		self.setAllowedIdentifications(None)	# default=accept all (None means all!)
 	def acceptHost(self,daemon,connection):
@@ -913,18 +943,20 @@ class BasicSSLValidator(DefaultConnValidator):
 
 
 #-------- Helper class for local storage.
-class LocalStorage:
+class LocalStorage(object):
 	def __init__(self):
 		self.caller=None
 
 #-------- TCPServer base class
 
 
-class TCPServer:
+class TCPServer(object):
 	def __init__(self, port, host='', threaded=_has_threading,prtcol='PYRO'):
 		self._ssl_server = 0
 		self.connections = []  # connection threads
 		self.initTLS=lambda tls: None  # default do-nothing func
+		if host:
+			socket.gethostbyname(host)  # validate hostname
 		try:
 			if prtcol=='PYROSSL':
 				try:
@@ -933,7 +965,12 @@ class TCPServer:
 					raise ProtocolError('SSL not available')
 				try:
 					self.ctx = SSL.Context('sslv23')
-					self.ctx.load_cert(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_SERVER_CERT))
+					if Pyro.config.PYROSSL_KEY:
+						keyfile = os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_KEY)
+					else:
+						keyfile = None
+					self.ctx.load_cert(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CERT),
+							   keyfile)
 					self.ctx.load_client_ca(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CA_CERT))
 					self.ctx.load_verify_info(os.path.join(Pyro.config.PYROSSL_CERTDIR, Pyro.config.PYROSSL_CA_CERT))
 					self.ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,10)
@@ -998,6 +1035,7 @@ class TCPServer:
 	def connectionHandler(self, conn):
 		# Handle the connection and all requests that arrive on it.
 		# This is only called in multithreading mode.
+		self.initTLS(self.getLocalStorage())
 		try:
 			if self.getAdapter().handleConnection(conn, self):
 				Log.msg('TCPServer','new connection ',conn, ' #conns=',len(self.connections))
@@ -1006,9 +1044,7 @@ class TCPServer:
 						if not conn.connected:
 							# connection has been closed in the meantime!
 							raise ConnectionClosedError()
-						ins,outs,exs=safe_select([conn],[],[],2)
-						if conn in ins or conn in exs:
-							self.handleInvocation(conn)
+						self.handleInvocation(conn)
 					except ConnectionClosedError:
 						# client went away. Exit immediately
 						self.removeConnection(conn)
@@ -1111,7 +1147,6 @@ class TCPServer:
 			thread=Thread(target=self.connectionHandler, args=(conn,))
 			thread.setDaemon(1)   # thread must exit at program termination.
 			thread.localStorage=LocalStorage()
-			self.initTLS(thread.localStorage)
 			self.connections.append(thread)
 			thread.start()
 		elif callback:
@@ -1153,14 +1188,17 @@ _selectfunction=select.select
 if os.name=="java":
 	from select import cpython_compatible_select as _selectfunction
 def safe_select(r,w,e,timeout=None):
+	delay=timeout
 	while True:
 		try:
-			if timeout is not None:
-				return _selectfunction(r,w,e,timeout)
+			# Make sure we don't delay longer than requested
+			start=time.time()
+			if delay is not None:
+				return _selectfunction(r,w,e,delay)
 			else:
 				return _selectfunction(r,w,e)
 		except select.error,x:
 			if x.args[0] == errno.EINTR or (hasattr(errno, 'WSAEINTR') and x.args[0] == errno.WSAEINTR):
-				pass
+				delay=max(0.0,time.time()-start)
 			else:
 				raise
